@@ -25,14 +25,9 @@ from bzrlib import errors
 from bzrlib.transport.local import LocalTransport
 from bzrlib.revision import NULL_REVISION, CURRENT_REVISION
 from bzrlib.tsort import merge_sort
-try:
-    from bzrlib.graph import (Graph, StackedParentsProvider)
-except ImportError:
-    from bzrlib.graph import (Graph,
-                    _StackedParentsProvider as StackedParentsProvider)
+from bzrlib.graph import (Graph, StackedParentsProvider)
     
 from bzrlib.bzrdir import BzrDir
-from bzrlib.inventory import Inventory
 from bzrlib.workingtree import WorkingTree
 from bzrlib.plugins.qbzr.lib.lazycachedrevloader import (load_revisions,
                                                          cached_revisions)
@@ -131,6 +126,8 @@ class RevisionInfo(object):
         return "%s <%s %s>" % (self.__class__.__name__, self.revno_str,
                               self.revid)
 
+paths_and_branches_err = "It is not possible to specify different file paths and different branches at the same time."
+
 class BranchLine(object):
     __slots__ = ["branch_id", "revs", "visible", "merges", "merged_by",
                  "color", "merge_depth", "expanded_by"]
@@ -148,6 +145,7 @@ class BranchLine(object):
     def __repr__(self):
         return "%s <%s>" % (self.__class__.__name__, self.branch_id)
 
+
 class LogGraphProvider(object):
     """Loads and computes revision and graph data for GUI log widgets."""
     
@@ -280,8 +278,6 @@ class LogGraphProvider(object):
         of locations strings, inputed by the user (such as at the command line.)
         
         """
-        paths_and_branches_err = "It is not possible to specify different file paths and different branches at the same time."
-        
         for location in locations:
             tree, br, repo, fp = \
                     BzrDir.open_containing_tree_branch_or_repository(location)
@@ -596,7 +592,11 @@ class LogGraphProvider(object):
         self.revid_rev = {}
         self.revno_rev = {}
         
+        self.max_mainline_revno = 0
+        
         for rev in self.revisions:
+            self.max_mainline_revno = max(self.max_mainline_revno, 
+                                          rev.revno_sequence[0])
             self.revid_rev[rev.revid] = rev
             self.revno_rev[rev.revno_sequence] = rev
         
@@ -628,9 +628,6 @@ class LogGraphProvider(object):
         
         """
         
-        self.start_branch_ids = []
-        """Branch ids that should be initialy visible"""
-        
         self.branch_ids = []
         """List of branch ids, sorted in the order that the branches will
         be shown, from left to right on the graph."""
@@ -645,7 +642,6 @@ class LogGraphProvider(object):
                 branch_line = self.branch_lines[rev.branch_id]
             
             if rev.revid in self.head_revids:
-                self.start_branch_ids.append(rev.branch_id)
                 branch_line.visible = True
             
             branch_line.revs.append(rev)
@@ -654,22 +650,14 @@ class LogGraphProvider(object):
         
         self.branch_ids = self.branch_lines.keys()
         
-        # Note: This greatly affects the layout of the graph.
-        def branch_id_cmp(x, y):
-            # Branch lines that have a tip (e.g. () - the main line) should be
-            # to the left of other branch lines.
-            is_start_x = x in self.start_branch_ids
-            is_start_y = y in self.start_branch_ids
-            if not is_start_x == is_start_y:
-                return -cmp(is_start_x, is_start_y)
+        def branch_id_sort_key(x):
+            merge_depth = self.branch_lines[x].merge_depth
             
+            # Note: This greatly affects the layout of the graph.
+            #
             # Branch line that have a smaller merge depth should be to the left
             # of those with bigger merge depths.
-            merge_depth_x = self.branch_lines[x].merge_depth
-            merge_depth_y = self.branch_lines[y].merge_depth
-            if not merge_depth_x == merge_depth_y:
-                return cmp(merge_depth_x, merge_depth_y)
-            
+            #
             # For branch lines that have the same parent in the mainline -
             # those with bigger branch numbers to be to the rights. E.g. for
             # the following dag, you want the graph to appear as on the left,
@@ -686,28 +674,34 @@ class LogGraphProvider(object):
             # 1.1.1 | B      |   B
             #       |/       | /
             # 1     A        A
-            if len(x) == 2 and len(y) == 2 and x[0] == y[0]:
-                return cmp(x[1], y[1])
-            
+            #
             # Otherwise, thoes with a greater mainline parent revno should
             # appear to the left.
-            return -cmp(x, y)
-
-        self.branch_ids.sort(branch_id_cmp)
+            
+            if len(x)==0:
+                return (merge_depth)
+            else:
+                return (merge_depth, -x[0], x[1])
+        
+        self.branch_ids.sort(key=branch_id_sort_key)
     
     def compute_merge_info(self):
         
-        def set_merged_by(rev, merged_by):
+        def set_merged_by(rev, merged_by, merged_by_rev, do_branches=False):
             if merged_by is not None:
+                if merged_by_rev is None:
+                    merged_by_rev = self.revisions[merged_by]
                 rev.merged_by = merged_by
-                self.revisions[merged_by].merges.append(rev.index)
-                branch_id = rev.branch_id
-                merged_by_branch_id = self.revisions[merged_by].branch_id
+                merged_by_rev.merges.append(rev.index)
                 
-                if not branch_id in self.branch_lines[merged_by_branch_id].merges:
-                    self.branch_lines[merged_by_branch_id].merges.append(branch_id)
-                if not merged_by_branch_id in self.branch_lines[branch_id].merged_by:
-                    self.branch_lines[branch_id].merged_by.append(merged_by_branch_id)
+                if do_branches:
+                    branch_id = rev.branch_id
+                    merged_by_branch_id = self.revisions[merged_by].branch_id
+                    
+                    if not branch_id in self.branch_lines[merged_by_branch_id].merges:
+                        self.branch_lines[merged_by_branch_id].merges.append(branch_id)
+                    if not merged_by_branch_id in self.branch_lines[branch_id].merged_by:
+                        self.branch_lines[branch_id].merged_by.append(merged_by_branch_id)
         
         for rev in self.revisions:
             
@@ -716,11 +710,11 @@ class LogGraphProvider(object):
             
             if len(parents) > 0:
                 if rev.branch_id == parents[0].branch_id:
-                    set_merged_by(parents[0], rev.merged_by)
+                    set_merged_by(parents[0], rev.merged_by, None)
             
             for parent in parents[1:]:
                 if rev.merge_depth<=parent.merge_depth:
-                    set_merged_by(parent, rev.index)
+                    set_merged_by(parent, rev.index, rev, do_branches=True)
         
     def compute_head_info(self):
         def get_revid_head(heads):
@@ -771,7 +765,7 @@ class LogGraphProvider(object):
                 ur.sort(key=lambda x: self.revid_rev[x].index)
 
     def load_filter_file_id_uses_inventory(self):
-        return self.has_dir and getattr(Inventory,"filter",None) is not None
+        return self.has_dir
     
     def load_filter_file_id(self):
         """Load with revisions affect the fileids
@@ -805,10 +799,9 @@ class LogGraphProvider(object):
                 if not self.load_filter_file_id_uses_inventory():
                     chunk_size = 500
                 else:
-                    chunk_size = 50
+                    chunk_size = 500
                 
                 for start in xrange(0, len(revids), chunk_size):
-                    text_keys = []
                     self.load_filter_file_id_chunk(repo, 
                             revids[start:start + chunk_size])
             
@@ -840,9 +833,14 @@ class LogGraphProvider(object):
                 for inv, revid in zip(
                             repo.iter_inventories(revids),
                             revids):
-                    filterted_inv = inv.filter(self.fileids)
-                    for path, entry in filterted_inv.entries():
+                    for path, entry in inv.iter_entries_by_dir(
+                                            specific_file_ids = self.fileids):
                         text_keys.append((entry.file_id, revid))
+                        if entry.kind == "directory":
+                            for rc_path, rc_entry in inv.iter_entries(from_dir = entry):
+                                text_keys.append((rc_entry.file_id, revid))
+                    
+                    self.update_ui()
                 
                 check_text_keys(text_keys)
         finally:
@@ -855,6 +853,8 @@ class LogGraphProvider(object):
     def get_revision_visible(self, index):
         """ Returns wether a revision is visible or not"""
         
+        if not index < len(self.revisions):
+            return False
         
         return self.revisions[index].f_index is not None
         #branch_id = self.revisions[index].branch_id
@@ -1321,7 +1321,6 @@ class LogGraphProvider(object):
                     parent_col_index = max(parent_col_index, 1)
                 
                 col_search_order = branch_line_col_search_order(parent_col_index) 
-                cur_cont_line = []
                 
                 if last_parent:
                     col_index = find_free_column(col_search_order,
@@ -1453,7 +1452,6 @@ class LogGraphProvider(object):
                         parent_branch_id in processed_branch_ids):
                         continue
                     
-                    collapse_parent = False
                     if parent.expanded_by == branch_id:
                         branch_ids.append((parent_branch_id, branch_id))
                     else:
